@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 1, or (at
+ * published by the Free Software Foundation; either version 2, or (at
  * your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
@@ -45,11 +45,18 @@
 #include "gjay.h"
 #include "analysis.h"
 #include "ipc.h"
+#include "i18n.h"
 
 #define BPM_BUF_SIZE    32*1024
 #define SHARED_BUF_SIZE sizeof(short int) * BPM_BUF_SIZE
 #define SLEEP_WHILE_IDLE 500
 
+#define BROKEN_VAL 100000000000000.0
+
+/* Multiplier for the frequency magnitude. 2.0 is close to the orginal value
+ * 5.0 gives a brighter display
+ */
+#define MAGS_MULTIPLIER 2.0
 
 typedef struct {
     FILE * f;
@@ -59,145 +66,119 @@ typedef struct {
 } wav_file;
 
 
-/* BPM globals */
-unsigned char *audio;
-unsigned long audiosize;
-unsigned long audiorate=2756;  /* Author states that 11025 is perfect
+
+#define AUDIO_RATE 2756UL      /* Author states that 11025 is perfect
                                   measure, but we can tolerate more
                                   lossiness for the sake of speed */
-unsigned long startbpm=120;
-unsigned long stopbpm=160;
-unsigned long startshift=0;
-unsigned long stopshift=0;
+#define START_BPM 120UL
+#define STOP_BPM 160UL
 
 
 /* Spectrum globals */
 #define MAX_FREQ 22500
 #define START_FREQ 100
 
-typedef unsigned long ulongT;
-typedef unsigned short ushortT;
+#define WINDOW_SIZE 1024
+#define STEP_SIZE 1024
 
 static char       * read_buffer;
 static int          read_buffer_size;
 static long         read_buffer_start;
 static long         read_buffer_end; /* same as file position */
-static int          window_size = 1024;
-static int          step_size = 1024;
 static gboolean     in_analysis = FALSE;  /* Are we currently analyizing a
                                          song? */
 static song       * analyze_song = NULL;
 static GList      * queue = NULL; 
 static GHashTable * queue_hash = NULL;
 static time_t       last_ping;
+static gchar * mp3_decoder;
+static gchar * ogg_decoder;
+static gchar * flac_decoder;
 
-void          analyze(char * fname);
-FILE *        inflate_to_wav (gchar * path, 
-                              song_file_type type);
-int           run_analysis     ( wav_file * wsfile,
+static FILE *     inflate_to_wav (const gchar * path, 
+                              const song_file_type type);
+
+static int           run_analysis     ( wav_file * wsfile,
                                  gdouble * freq_results,
                                  gdouble * volume_diff,
                                  gdouble * bpm_result );
-unsigned long bpm_phasefit     ( long i );
-int           freq_read_frames ( wav_file * wsfile, 
+static unsigned long bpm_phasefit     ( const long i,
+    const unsigned char const *audio,
+    const unsigned long audiosize);
+static int           freq_read_frames ( wav_file * wsfile, 
                                  int start, 
                                  int length, 
                                  void *data );
-void          send_ui_percent        ( int percent );
-void          send_analyze_song_name ( void );
-gboolean      daemon_idle       ( gpointer data );
+static void          send_ui_percent        ( int percent );
+static void          send_analyze_song_name ( void );
 static void   write_queue       ( void );
+static void kill_signal (int sig);
+static void analysis_daemon(void);
+gboolean      daemon_idle       ( gpointer data );
+static void analyze( const char * fname, const gboolean result_to_stdout);
 
-void analysis_daemon(void) {
-    GIOChannel * ui_io;
-    GMainLoop * loop;
-    char buffer[BUFFER_SIZE];
-    gchar * file;
-    FILE * f;
-    
-    /* Nice the analysis */
-    setpriority(PRIO_PROCESS, getpid(), 19);
-    
-    in_analysis = FALSE;
-    loop = g_main_new(FALSE);
-    last_ping = time(NULL);
-    queue_hash = g_hash_table_new(g_str_hash, g_str_equal);
-    
-    /* Read analysis queue, if any */
-    snprintf(buffer, BUFFER_SIZE, "%s/%s/%s", getenv("HOME"), 
-             GJAY_DIR, GJAY_QUEUE);
-    f = fopen(buffer, "r");
-    if (f) {
-        while (!feof(f)) {
-            read_line(f, buffer, BUFFER_SIZE);
-            if (strlen(buffer) &&!g_hash_table_lookup(queue_hash, buffer)) {
-                file = g_strdup(buffer);
-                g_hash_table_insert(queue_hash, file, (void *) 1);
-                queue = g_list_append(queue, file);
-            }
-        }
-        fclose(f);
-    }
-    
-    ui_io = g_io_channel_unix_new (ui_pipe_fd);
-    g_io_add_watch (ui_io,
-                    G_IO_IN,
-                    ui_pipe_input,
-                    loop);
-
-    g_idle_add (daemon_idle, loop);
-    // FIXME: add G_IO_HUP watcher
-
-    g_main_run(loop);
+void
+run_as_analyze_detached  ( const char * analyze_detached_fname)
+{
+  if (access(analyze_detached_fname, R_OK) != 0)
+  {
+    fprintf(stderr, _("Song file %s not found\n"), analyze_detached_fname);
+    exit(1);
+  }
+  analyze(analyze_detached_fname, TRUE);
 }
 
+void run_as_daemon(void)
+{
+  gchar *path;
+  FILE *fp;
 
-gboolean daemon_idle (gpointer data) {
-    gchar * file;
-
-    if ((mode != DAEMON_DETACHED) && 
-        (time(NULL) - last_ping > DAEMON_ATTACH_FREAKOUT)) {
-        if (verbosity)
-            printf("Daemon appears to have been orphaned. Quitting.\n");
-        g_main_quit((GMainLoop *) data);
-    } 
-
-    if (mode == DAEMON_INIT) {
-        usleep(SLEEP_WHILE_IDLE);
-        return TRUE;
-    }
-
-    if (in_analysis)
-        return TRUE;
+  /* Write pid to ~/.gjay/gjay.pid */
+  path = g_strdup_printf("%s/%s/%s", g_get_home_dir(),
+      GJAY_DIR, GJAY_PID);
+  if ((fp = fopen(path, "w")) == NULL) {
+    g_error(_("Cannot open %s for writing\n"), path);
+    exit(1);
+  }
+  g_free(path);
+  fprintf(fp, "%d", getpid());
+  fclose(fp);
     
-    if (queue) {
-        file = g_list_first(queue)->data;
-        analyze(file);
-        g_hash_table_remove(queue_hash, file);
-        queue = g_list_remove(queue, file);
-        g_free(file);
-        
-        write_queue();
-    }
+  signal(SIGTERM, kill_signal);
+  signal(SIGKILL, kill_signal);
+  signal(SIGINT,  kill_signal);
     
-    if (queue)
-        return TRUE;
-
-    if (mode == DAEMON_DETACHED) {
-        if (verbosity)
-            printf("Analysis daemon done.\n");
-        g_main_quit((GMainLoop *) data);
-    }
-    return FALSE;
+  analysis_daemon();
+    
+    /* Daemon cleans up pipes on quit */
+  close(daemon_pipe_fd);
+  close(ui_pipe_fd);
+  unlink(daemon_pipe);
+  unlink(ui_pipe);
+  rmdir(gjay_pipe_dir);
+  g_free(gjay_pipe_dir);
 }
 
+/**
+ * When the daemon receives a kill signal, delete ~/.gjay/gjay.pid
+ */
+static void kill_signal (int sig) {
+  gchar *path;
+
+  /* Write pid to ~/.gjay/gjay.pid */
+  path = g_strdup_printf("%s/%s/%s", g_get_home_dir(),
+      GJAY_DIR, GJAY_PID);
+  unlink(path); 
+  g_free(path);
+  exit(0);
+}
 
 static void add_file_to_queue ( char * fname) {
     char queue_fname[BUFFER_SIZE], buffer[BUFFER_SIZE], * str;
     FILE * f_queue, * f_add;
         
     if (verbosity > 1) {
-        printf("Adding '%s' to analysis queue\n", fname);
+        printf(_("Adding '%s' to analysis queue\n"), fname);
     }
     snprintf(queue_fname, BUFFER_SIZE, "%s/%s/%s", getenv("HOME"), 
              GJAY_DIR, GJAY_QUEUE);
@@ -271,21 +252,21 @@ gboolean ui_pipe_input (GIOChannel *source,
         break;
     case ACK:
         if (verbosity > 1)
-            printf("Daemon received ack\n");
+            printf(_("Daemon received ack\n"));
         // No need for action
         break;
     case UNLINK_DAEMON_FILE:
-        if (verbosity > 1)
-            printf("Deleting daemon pid file\n");
         snprintf(buffer, BUFFER_SIZE, "%s/%s/%s", 
                  getenv("HOME"), GJAY_DIR, GJAY_DAEMON_DATA);
+        if (verbosity > 1)
+            printf(_("Deleting daemon pid file '%s'\n"), buffer);
         unlink(buffer);
         break;
     case CLEAR_ANALYSIS_QUEUE:
-        if (verbosity > 1)
-            printf("Daemon is clearing out analysis queue, file\n");
         snprintf(buffer, BUFFER_SIZE, "%s/%s/%s", getenv("HOME"), 
                  GJAY_DIR, GJAY_QUEUE);
+        if (verbosity > 1)
+            printf(_("Daemon is clearing out analysis queue, deleting file '%s'\n"), buffer);
         unlink(buffer);
         for (list = g_list_first(queue); list; list = g_list_next(list)) 
             g_free(list->data);
@@ -300,7 +281,7 @@ gboolean ui_pipe_input (GIOChannel *source,
         buffer[len] = '\0';
         file = buffer + sizeof(ipc_type);
         if (verbosity > 1) 
-            printf("Queuing file %s\n", file);
+            printf(_("Daemon queuing song file '%s'\n"), file);
         add_file_to_queue(file);
         g_idle_add (daemon_idle, data);
         unlink(file);
@@ -308,7 +289,7 @@ gboolean ui_pipe_input (GIOChannel *source,
     case DETACH: 
         if (mode == DAEMON) {
             if (verbosity)
-                printf("Detaching daemon\n");
+                printf(_("Detaching daemon\n"));
             mode = DAEMON_DETACHED;
             g_idle_add (daemon_idle, (GMainLoop * ) data);
         }
@@ -316,7 +297,7 @@ gboolean ui_pipe_input (GIOChannel *source,
     case ATTACH:
         if (mode != DAEMON) {
             if (verbosity)
-                printf("Attaching to daemon\n");
+                printf(_("Attaching to daemon\n"));
             mode = DAEMON;
             if(in_analysis && analyze_song)
                 send_analyze_song_name();
@@ -326,20 +307,23 @@ gboolean ui_pipe_input (GIOChannel *source,
     case QUIT_IF_ATTACHED:
         if (mode == DAEMON) {
             if (verbosity)
-                printf("Daemon Quitting\n");
+                printf(_("Daemon Quitting\n"));
             g_main_quit ((GMainLoop * ) data);
-            exit(0);
         }
         break;
     default:
         // Do nothing
+        break;
     }
     return TRUE;
 }
 
 
 
-void analyze(char * fname) {
+static void
+analyze(const char * fname,            /* File to analyze */
+        const gboolean result_to_stdout) /* Write result to stdout? */
+{
     FILE * f;
     gchar * utf8;
     wav_file wsfile;
@@ -348,17 +332,22 @@ void analyze(char * fname) {
     gdouble freq[NUM_FREQ_SAMPLES], volume_diff, analyze_bpm;
     gboolean is_song;
     song_file_type type;
-    time_t t;
+    time_t t=0;
 
     analyze_song = NULL;
     in_analysis = TRUE;
+    
     send_ui_percent(0);
+    if (fname == NULL || fname[0] == '\0') {
+      in_analysis = FALSE;
+      return;
+    }
     
     if (access(fname, R_OK) != 0) {
         /* File ain't there! The UI thread will check for non-existant
            files periodically. */
         if (verbosity)
-            fprintf(stderr, "File not found\n");
+            g_warning(_("analyze(): File '%s' cannot be read\n"),fname);
         in_analysis = FALSE;
         return;
     }
@@ -381,11 +370,11 @@ void analyze(char * fname) {
 
     if (!is_song) {
         if (verbosity)
-            fprintf(stderr, "File not song\n");
+          g_warning(_("File '%s' is not a recognised song.\n"),fname);
         in_analysis = FALSE;
         return;
     }
-
+    
     send_analyze_song_name();
     send_ipc_text(daemon_pipe_fd, ANIMATE_START, analyze_song->path);
 
@@ -397,14 +386,14 @@ void analyze(char * fname) {
     wsfile.header.data_length = (MAX(1, analyze_song->length - 1)) * wsfile.header.byte_p_sec;
 
     if (verbosity) {
-        printf("Analyzing %s\n", fname);
+        printf(_("Analyzing song file '%s'\n"), fname);
         t = time(NULL);
     }
 
     result = run_analysis(&wsfile, freq, &volume_diff, &analyze_bpm); 
 
     if (verbosity) 
-        printf("Analysis took %ld seconds\n", time(NULL) - t);
+        printf(_("Analysis took %ld seconds\n"), time(NULL) - t);
     
     send_ui_percent(0);
 
@@ -428,8 +417,12 @@ void analyze(char * fname) {
 
     send_ipc(daemon_pipe_fd, ANIMATE_STOP);
     send_ipc_text(daemon_pipe_fd, STATUS_TEXT, "Idle");
-    
-    result = append_daemon_file(analyze_song);
+
+    if (result_to_stdout) {
+        write_song_data(stdout, analyze_song);
+    } else {
+        result = append_daemon_file(analyze_song);
+    }
     if (result >= 0) 
         send_ipc_int(daemon_pipe_fd, ADDED_FILE, result);
     delete_song(analyze_song);
@@ -438,38 +431,86 @@ void analyze(char * fname) {
     return;
 }
 
+static void
+check_decoders(void)
+{
+  if (ogg_decoder == NULL)
+    ogg_decoder = g_find_program_in_path(OGG_DECODER_APP);
 
-FILE * inflate_to_wav ( gchar * path,
-                        song_file_type type) {
-    char buffer[BUFFER_SIZE];
-    char quoted_path[BUFFER_SIZE];
-    FILE * f;
+  if (mp3_decoder == NULL)
+    mp3_decoder = g_find_program_in_path(MP3_DECODER_APP1);
+  if (mp3_decoder == NULL)
+    mp3_decoder = g_find_program_in_path(MP3_DECODER_APP2);
 
-    quote_path(quoted_path, BUFFER_SIZE, path);
-    switch (type) {
+  if (flac_decoder == NULL)
+    flac_decoder = g_find_program_in_path(FLAC_DECODER_APP);
+
+}
+
+static FILE *
+inflate_to_wav (const gchar * path, const song_file_type type)
+{
+  gchar *cmdline;
+  gchar *quoted_path;
+  FILE *fp;
+
+  quoted_path = g_shell_quote(path);
+  if (mp3_decoder == NULL)
+    check_decoders();
+
+  switch (type) {
     case OGG:
-        snprintf(buffer, BUFFER_SIZE, "ogg123 \'%s\' -d wav -f - 2> /dev/null",
-                 quoted_path);
-        break;
+      if (ogg_decoder == NULL) {
+        if (verbosity)
+          g_warning(_("Unable to decode '%s' as no ogg decoder found"), path);
+        return NULL;
+      }
+      cmdline = g_strdup_printf("%s %s -d wav -f - 2> /dev/null",
+          ogg_decoder,
+          quoted_path);
+      break;
     case MP3:
-        snprintf(buffer, BUFFER_SIZE, "mpg321 -b 10000 \'%s\' -w - 2> /dev/null",
-                 quoted_path);
-        break;
+      if (mp3_decoder == NULL) {
+        if (verbosity)
+          g_warning(_("Unable to decode '%s' as no mp3 decoder found"), path);
+        return NULL;
+      }
+      cmdline = g_strdup_printf("%s -b 10000 %s -w - 2> /dev/null",
+          mp3_decoder,
+          quoted_path);
+      break;
+    case FLAC:
+      if (flac_decoder == NULL) {
+        if (verbosity)
+          g_warning(_("Unable to decode '%s' as no flac decoder found"), path);
+        return NULL;
+      }
+      cmdline = g_strdup_printf("%s -d -c %s 2> /dev/null",
+          flac_decoder,
+          quoted_path);
+      break;
     case WAV:
     default:
-        return fopen(path, "r");
-    }
-    if (!(f = popen(buffer, "r"))) {
-        fprintf(stderr, "Unable to run %s\n", buffer);
-        return NULL;
-    } 
-    return f;
+      g_free(quoted_path);
+      return fopen(path, "r");
+  }
+  g_free(quoted_path);
+  //fprintf(stderr,"Decoding with:\n ->%s\n->%s", cmdline,path);
+  if (!(fp = popen(cmdline, "r"))) {
+    g_error(_("Unable to run decoder command '%s'\n"), cmdline);
+    g_free(cmdline);
+      return NULL;
+  } 
+  g_free(cmdline);
+  return fp;
 }
 
 
 /* Swap the byte order of wav header. Wavs are stored little-endian, so this
    is necessary when using on big-endian (e.g. PPC) machines */
-void wav_header_swab(waveheaderstruct * header) {
+void
+wav_header_swab(waveheaderstruct * header)
+{
     header->length = le32_to_cpu(header->length);
     header->length_chunk = le32_to_cpu(header->length_chunk);
     header->data_length = le32_to_cpu(header->data_length);
@@ -481,30 +522,6 @@ void wav_header_swab(waveheaderstruct * header) {
     header->bit_p_spl = le16_to_cpu(header->bit_p_spl);
 }
 
-
-
-/* Escape ' char in path */
-int quote_path(char *buf, size_t bufsiz, const char *path) {
-    int in, out = 0;
-    const char *quote = "\'\\\'\'"; /* a quoted quote character */
-    for (in = 0; out < bufsiz && path[in] != '\0'; in++) {
-        if (path[in] == '\'') {
-            if (out + strlen(quote) > bufsiz) {
-                break;
-            } else {
-                memcpy(&buf[out], quote, strlen(quote)); 
-                out += strlen(quote);
-            }
-        } else {
-            buf[out++] = path[in];
-        }
-    }
-    if (out < bufsiz)
-        buf[out++] = '\0';
-    return out;
-}
-
-
 /**
  * This is the big, bad analysis algorithm. To make things REALLY complicated,
  * I'm interspersing two algorithsm -- BPM and frequency analysis -- such that
@@ -516,55 +533,62 @@ int quote_path(char *buf, size_t bufsiz, const char *path) {
  *
  * Any ugliness is the fault of my munging. 
  */
-int run_analysis  (wav_file * wsfile,
+static int
+run_analysis  (wav_file * wsfile,
                    gdouble * freq_results,
                    gdouble * volume_diff,
-                   gdouble * bpm_result ) {
+                   gdouble * bpm_result )
+{
     int16_t *freq_data;
     double *ch1 = NULL, *ch2 = NULL, *mags = NULL;
     long i, j, k, bin;
-    double ch1max = 0, ch2max = 0;
     double *total_mags;
     double sum, frame_sum, max_frame_sum, g_factor, freq, g_freq;
     signed short buffer[BPM_BUF_SIZE];
     long count, pos, h, redux, startpos, num_frames;
+    unsigned long startshift = 0, stopshift = 0;
+    unsigned char *audio;
+    unsigned long audiosize;
 
     if (wsfile->header.modus != 1 && wsfile->header.modus != 2) {
+      if (verbosity > 2)
+        g_warning(_("File is not a (converted) wav file. Modus is supposed to be 1 or 2 but is %d\n"), wsfile->header.modus);
         // Not a wav file
         return FALSE;
     }
     
     if (wsfile->header.byte_p_spl / wsfile->header.modus != 2) {
+      if (verbosity > 2)
+        g_warning(_("File is not a 16-bit stream\n"));
         // Not 16-bit
         return FALSE;
     }
 
     /* Spectrum set-up */    
-    read_buffer_size = window_size*4;
+    read_buffer_size = WINDOW_SIZE*4;
     read_buffer = malloc(read_buffer_size);
     read_buffer_start = 0;
     read_buffer_end = read_buffer_size;
-    freq_data = (int16_t*) malloc (window_size * wsfile->header.byte_p_spl);
-    mags = (double*) malloc (window_size / 2 * sizeof (double));
-    total_mags = (double*) malloc (window_size / 2 * sizeof (double));
-    memset (total_mags, 0x00, window_size / 2 * sizeof (double));
+    freq_data = (int16_t*) g_malloc0 (WINDOW_SIZE * wsfile->header.byte_p_spl);
+    mags = (double*) g_malloc0 (WINDOW_SIZE / 2 * sizeof (double));
+    total_mags = (double*) g_malloc0 (WINDOW_SIZE / 2 * sizeof (double));
+    //memset (total_mags, 0x00, WINDOW_SIZE / 2 * sizeof (double));
     memset (freq_results, 0x00, NUM_FREQ_SAMPLES * sizeof(double));
-    ch1 = (double*) malloc (window_size * sizeof (double));
+    ch1 = (double*) g_malloc0 (WINDOW_SIZE * sizeof (double));
     if (wsfile->header.modus == 2)
-        ch2 = (double*) malloc (window_size * sizeof (double));
+        ch2 = (double*) g_malloc0 (WINDOW_SIZE * sizeof (double));
     num_frames = 0;
     max_frame_sum = 0;
     sum = 0;
 
     /* BPM set-up */
     audiosize = wsfile->header.data_length;
-    audiosize/=(4*(44100/audiorate));
-    audio=malloc(audiosize+1);
+    audiosize/=(4*(44100/AUDIO_RATE));
+    audio= g_malloc0(audiosize+1);
     assert(audio);
     pos=0;
     startpos = pos;
 
-
     /* Read the first chunk of the file into the shared buffer */
     fread(wsfile->buffer, 1, SHARED_BUF_SIZE, wsfile->f);
     wsfile->seek = SHARED_BUF_SIZE;
@@ -576,9 +600,9 @@ int run_analysis  (wav_file * wsfile,
     /* In this main loop, we read the entire file. Most of this loop 
      * represents the spectrum algorithm; the marked tight loop is the bpm
      * algorithm. */
-    for (i = -window_size; i < window_size + (int)(wsfile->header.data_length / wsfile->header.byte_p_spl); i += step_size) {
+    for (i = -WINDOW_SIZE; i < WINDOW_SIZE + (int)(wsfile->header.data_length / wsfile->header.byte_p_spl); i += STEP_SIZE) {
 
-        freq_read_frames (wsfile, i, window_size, (char *)freq_data);
+        freq_read_frames (wsfile, i, WINDOW_SIZE, (char *)freq_data);
 
         if (wsfile->freq_seek == SHARED_BUF_SIZE) {
             /* At end of buffer. Read some more data. */
@@ -591,62 +615,70 @@ int run_analysis  (wav_file * wsfile,
             send_ui_percent(wsfile->seek/(wsfile->header.data_length / 70));
             
             /* BPM loop */
-            for (h=0;h<count/2;h+=2*(44100/audiorate))
+            for (h=0;h<count/2;h+=2*(44100/AUDIO_RATE))
             {
                 signed long int left, right,mean;
                 left=abs(buffer[h]);
                 right=abs(buffer[h+1]);
                 mean=(left+right)/2;
                 redux=abs(mean)/128;
-                if (pos+h/(2*(44100/audiorate))>=audiosize) break;
-                assert(pos+h/(2*(44100/audiorate))<audiosize);
-                audio[pos+h/(2*(44100/audiorate))]=(unsigned char)redux;
+                if (pos+h/(2*(44100/AUDIO_RATE))>=audiosize) break;
+                assert(pos+h/(2*(44100/AUDIO_RATE))<audiosize);
+                audio[pos+h/(2*(44100/AUDIO_RATE))]=(unsigned char)redux;
             }
-            pos+=count/(4*(44100/audiorate));
+            pos+=count/(4*(44100/AUDIO_RATE));
         }
         
         /* The rest of this loop is spectrum analysis */
         if (wsfile->header.modus == 1) {
-            for (j = 0; j < window_size; j++)
+            for (j = 0; j < WINDOW_SIZE; j++)
                 ch1 [j] = (int16_t)le16_to_cpu(freq_data[j]);
+            gsl_fft_real_radix2_transform (ch1, 1, WINDOW_SIZE);
             
         } else {
-            for (j = 0, k = 0; k < window_size; k++, j+=2) {
+            for (j = 0, k = 0; k < WINDOW_SIZE; k++, j+=2) {
                 ch1[k] = (double)((int16_t)le16_to_cpu(freq_data[j]));
                 ch2[k] = (double)((int16_t)le16_to_cpu(freq_data[j+1]));
             } 
-            gsl_fft_real_radix2_transform (ch2, 1, window_size);
-        }
-        gsl_fft_real_radix2_transform (ch1, 1, window_size);
-        
-        mags [0] = fabs (ch1 [0]);
-        
-        for (j = 0; j < window_size / 2; j++) {
-            mags [j] = sqrt (ch1 [j] * ch1 [j] + ch1 [window_size - j] * ch1 [window_size - j]);
-            
-            if (mags [j] > ch1max)
-                ch1max = mags [j];
+            gsl_fft_real_radix2_transform (ch1, 1, WINDOW_SIZE);
+            gsl_fft_real_radix2_transform (ch2, 1, WINDOW_SIZE);
         }
         
+//        mags [0] = fabs (ch1 [0]);
+        
+        for (j = 0; j < WINDOW_SIZE / 2; j++) {
+	    double square_this = ch1 [j] * ch1 [j] + ch1 [WINDOW_SIZE - j] * ch1 [WINDOW_SIZE - j];
+	    if (square_this > 0.0 && square_this < BROKEN_VAL)
+        	mags [j] = sqrt (square_this);
+    	    else
+    		mags [j] = 0;
+        }
+
         /* Add magnitudes */
-        for (k = 0; k < window_size / 2; k++) 
+        for (frame_sum = 0, k = 0; k < WINDOW_SIZE / 2; k++) {
             total_mags[k] += mags[k];
-        
+            frame_sum += mags[k];
+        }
+
+        max_frame_sum = MAX(frame_sum, max_frame_sum);
+        sum += frame_sum;
+        num_frames++;
+
         if (wsfile->header.modus == 2) {
-            mags [0] = fabs (ch2 [0]);
-            
-            for (j = 0; j < window_size / 2; j++) {
-                mags [j] = sqrt (ch2 [j] * ch2 [j] + ch2 [window_size - j] * ch2 [window_size - j]);
-                
-                if (mags [j] > ch2max)
-                    ch2max = mags [j];
+            for (j = 0; j < WINDOW_SIZE / 2; j++) {
+		double square_this = ch2 [j] * ch2 [j] + ch2 [WINDOW_SIZE - j] * ch2 [WINDOW_SIZE - j];
+		if ( square_this > 0 && square_this < BROKEN_VAL)
+        	    mags [j] = sqrt (square_this);
+    		else
+    		    mags [j] = 0;
             }
 
             /* Add magnitudes */            
-            for (frame_sum = 0, k = 0; k < window_size / 2; k++) {
+            for (frame_sum = 0, k = 0; k < WINDOW_SIZE / 2; k++) {
                 total_mags[k] += mags[k];
                 frame_sum += mags[k];
             }
+        
             max_frame_sum = MAX(frame_sum, max_frame_sum);
             sum += frame_sum;
             num_frames++;
@@ -656,15 +688,15 @@ int run_analysis  (wav_file * wsfile,
     /* Finish analysis... */
     *volume_diff =  max_frame_sum / (sum / (gdouble) num_frames);
 
-    for (k = 0; k < window_size / 2; k++) 
-        total_mags[k] = total_mags[k] / sum;
+    for (k = 0; k < WINDOW_SIZE / 2; k++) 
+        total_mags[k] = (total_mags[k] / sum) * MAGS_MULTIPLIER;
     
     g_freq = START_FREQ;
     g_factor = exp( log(MAX_FREQ/START_FREQ) / NUM_FREQ_SAMPLES);
     bin = 0;
-    for (k = 0; k < window_size / 2; k++) {
+    for (k = 0; k < WINDOW_SIZE / 2; k++) {
         /* Determine which frequency band this sample falls into */
-        freq = (k * MAX_FREQ ) / (window_size / 2);
+        freq = (k * MAX_FREQ ) / (WINDOW_SIZE / 2);
         if (freq > g_freq) {
             bin = MIN(bin + 1, NUM_FREQ_SAMPLES - 1);
             g_freq *= g_factor;
@@ -673,22 +705,23 @@ int run_analysis  (wav_file * wsfile,
     }
 
     if (verbosity > 1) {
-        printf("Frequencies: \n");
+        printf(_("Frequencies: \n"));
         for (k = 0; k < NUM_FREQ_SAMPLES; k++) 
             printf("%f ", freq_results[k]);
         printf("\n");
     }
 
     /* Complete BPM analysis */
-    stopshift=audiorate*60*4/startbpm;
-    startshift=audiorate*60*4/stopbpm;
+    stopshift=AUDIO_RATE*60*4/START_BPM;
+    startshift=AUDIO_RATE*60*4/STOP_BPM;
     {
 	unsigned long foutat[stopshift-startshift];
-	unsigned long fout, minimumfout=0, maximumfout,minimumfoutat,left,right;
+	unsigned long fout, minimumfout=0, maximumfout,minimumfoutat=ULONG_MAX,
+                left,right;
         memset(&foutat,0,sizeof(foutat));
 	for(h=startshift;h<stopshift;h+=50)
         {
-            fout=bpm_phasefit(h);
+            fout=bpm_phasefit(h, audio, audiosize);
             foutat[h-startshift]=fout;
             if (minimumfout==0) maximumfout=minimumfout=fout;
             if (fout<minimumfout) 
@@ -707,7 +740,7 @@ int run_analysis  (wav_file * wsfile,
 	if ( right > stopshift ) 
             right = stopshift;
 	for(h=left; h<right; h++) {
-            fout=bpm_phasefit(h);
+            fout=bpm_phasefit(h, audio, audiosize);
             foutat[h-startshift]=fout;
             if (minimumfout==0) maximumfout=minimumfout=fout;
             if (fout<minimumfout) 
@@ -727,24 +760,21 @@ int run_analysis  (wav_file * wsfile,
                 fout-=minimumfout;
             }
         }
-        *bpm_result = 4.0*(double)audiorate*60.0/(double)minimumfoutat;
+        *bpm_result = 4.0*(double)AUDIO_RATE*60.0/(double)minimumfoutat;
         if (verbosity > 1) 
-            printf("BPM: %f\n", *bpm_result);
+            printf(_("BPM: %f\n"), *bpm_result);
     }
-    free (audio);
-    free (freq_data);
-    free (mags);
-    free (total_mags);
-    free (read_buffer);
+    g_free (audio);
+    g_free (freq_data);
+    g_free (mags);
+    g_free (total_mags);
+    g_free (read_buffer);
     return TRUE;
 }
 
 
-
-int freq_read_frames (wav_file * wsfile,
-                      int start, 
-                      int length, 
-                      void *data)
+static int
+freq_read_frames (wav_file * wsfile, int start, int length, void *data)
 {
     int realstart = start;
     int reallength = length;
@@ -800,7 +830,9 @@ int freq_read_frames (wav_file * wsfile,
 }
 
 
-unsigned long bpm_phasefit(long i)
+static unsigned long
+bpm_phasefit(const long i, const unsigned char const *audio,
+    const unsigned long audiosize)
 {
    long c,d;
    unsigned long mismatch=0;
@@ -817,7 +849,9 @@ unsigned long bpm_phasefit(long i)
 
 
 
-void send_ui_percent (int percent) {
+static void
+send_ui_percent (int percent)
+{
     static int old_percent = -1;
 
     if (old_percent == percent)
@@ -836,8 +870,9 @@ void send_ui_percent (int percent) {
 }
 
 
-
-void send_analyze_song_name ( void ) {
+static void
+send_analyze_song_name ( void )
+{
     char buffer[BUFFER_SIZE];
     if (!analyze_song)
         return;
@@ -851,3 +886,88 @@ void send_analyze_song_name ( void ) {
     strncpy(buffer + 60, "...\0", 4);
     send_ipc_text(daemon_pipe_fd, STATUS_TEXT, buffer);
 }
+
+
+
+static void
+analysis_daemon(void) {
+    GIOChannel * ui_io;
+    GMainLoop * loop;
+    char buffer[BUFFER_SIZE];
+    gchar * file;
+    FILE * f;
+    
+    /* Nice the analysis */
+    setpriority(PRIO_PROCESS, getpid(), 19);
+    
+    in_analysis = FALSE;
+    loop = g_main_new(FALSE);
+    last_ping = time(NULL);
+    queue_hash = g_hash_table_new(g_str_hash, g_str_equal);
+
+    /* Read analysis queue, if any */
+    snprintf(buffer, BUFFER_SIZE, "%s/%s/%s", getenv("HOME"), 
+             GJAY_DIR, GJAY_QUEUE);
+    f = fopen(buffer, "r");
+    if (f) {
+        while (!feof(f)) {
+            read_line(f, buffer, BUFFER_SIZE);
+            if (strlen(buffer) &&!g_hash_table_lookup(queue_hash, buffer)) {
+                file = g_strdup(buffer);
+                g_hash_table_insert(queue_hash, file, (void *) 1);
+                queue = g_list_append(queue, file);
+            }
+        }
+        fclose(f);
+    }
+    ui_io = g_io_channel_unix_new (ui_pipe_fd);
+    g_io_add_watch (ui_io,
+                    G_IO_IN,
+                    ui_pipe_input,
+                    loop);
+    g_idle_add (daemon_idle, loop);
+    // FIXME: add G_IO_HUP watcher
+
+    g_main_run(loop);
+}
+
+
+gboolean daemon_idle (gpointer data) {
+    gchar * file;
+
+    if ((mode != DAEMON_DETACHED) && 
+        (time(NULL) - last_ping > DAEMON_ATTACH_FREAKOUT)) {
+        if (verbosity)
+            printf(_("Daemon appears to have been orphaned. Quitting.\n"));
+        g_main_quit((GMainLoop *) data);
+    } 
+
+    if (mode == DAEMON_INIT) {
+        usleep(SLEEP_WHILE_IDLE);
+        return TRUE;
+    }
+
+    if (in_analysis)
+        return TRUE;
+    
+    if (queue) {
+        file = g_list_first(queue)->data;
+        analyze(file, FALSE);
+        g_hash_table_remove(queue_hash, file);
+        queue = g_list_remove(queue, file);
+        g_free(file);
+        
+        write_queue();
+    }
+    
+    if (queue)
+        return TRUE;
+
+    if (mode == DAEMON_DETACHED) {
+        if (verbosity)
+            printf(_("Analysis daemon done.\n"));
+        //g_main_quit((GMainLoop *) data);
+    }
+    return FALSE;
+}
+
