@@ -17,6 +17,10 @@
  * USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h> 
@@ -27,10 +31,13 @@
 #include <math.h> 
 #include <ctype.h>
 #include "gjay.h"
-#include "gjay_xmms.h"
+#include "gjay_audacious.h"
 #include "analysis.h"
 #include "mp3.h"
 #include "vorbis.h"
+#include "flac.h"
+#include "ui.h"
+#include "i18n.h"
 
 
 typedef enum {
@@ -90,17 +97,7 @@ typedef struct {
 } song_parse_state;
 
 
-
-GList      * songs;       /* List of song ptrs  */
-GList      * not_songs;   /* List of char *, UTF8 encoded */
-gboolean     songs_dirty;
-
-GHashTable * song_name_hash; 
-GHashTable * song_inode_dev_hash;
-GHashTable * not_song_hash;
-
-
-static void     write_song_data     ( FILE * f, song * s );
+static gdouble song_mass   ( song * s );
 static void     write_not_song_data ( FILE * f, gchar * path );
 static gboolean read_song_file_type ( char * path, 
                                       song_file_type type,
@@ -335,7 +332,7 @@ void file_info ( gchar    * path,
     *album = NULL;
 
     if (verbosity > 1) {
-        printf("Scanning %s...\n", path);
+        printf(_("Scanning '%s'...\n"), path);
     }
 
     latin1_path = strdup_to_latin1(path);
@@ -347,30 +344,58 @@ void file_info ( gchar    * path,
     *dev = buf.st_dev;
     *inode = buf.st_ino;
 
-    for (*type = OGG; *type < SONG_FILE_TYPE_LAST; (*type)++) {
-        if (read_song_file_type(latin1_path, *type, 
-                                length, title, artist, album)) {
-            *is_song = TRUE;
-            g_free(latin1_path);
-            return;
-        }
+#ifdef HAVE_VORBIS_VORBISFILE_H
+    if (gjay->ogg_supported && read_ogg_file_type(latin1_path, length, title, artist, album) == TRUE)
+    {
+      *is_song = TRUE;
+      *type = OGG;
+      g_free(latin1_path);
+      return;
     }
+#endif /* HAVE_VORBIS_VORBISFILE_H */
+
+    if (read_mp3_file_type(latin1_path, length, title, artist, album) == TRUE)
+    {
+      *is_song = TRUE;
+      *type = MP3;
+      g_free(latin1_path);
+      return;
+    }
+    *type = WAV;
+    if (read_song_file_type(latin1_path, *type, 
+                                length, title, artist, album)) {
+      *is_song = TRUE;
+      g_free(latin1_path);
+      return;
+    }
+
+#ifdef HAVE_FLAC_METADATA_H
+    if (gjay->flac_supported && read_flac_file_type(latin1_path, length, title, artist, album) == TRUE)
+    {
+      *is_song = TRUE;
+      *type = FLAC;
+      g_free(latin1_path);
+      return;
+    }
+#endif /* HAVE_FLAC_METADATA_H */
+
     g_free(latin1_path);
 }
 
 
 void write_data_file(void) {
-    char buffer[BUFFER_SIZE], buffer_temp[BUFFER_SIZE];
+    gchar *tmp_filename, *data_filename;
     FILE * f;
     GList * llist, * w_songs = NULL;
     song * s;
 
-    snprintf(buffer, BUFFER_SIZE, "%s/%s/%s", getenv("HOME"), 
-             GJAY_DIR, GJAY_FILE_DATA);
-    snprintf(buffer_temp, BUFFER_SIZE, "%s_temp", buffer);
-
+    tmp_filename = g_strdup_printf("%s/%s/%s_temp",
+        g_get_home_dir(), GJAY_DIR, GJAY_FILE_DATA);
+    data_filename = g_strdup_printf("%s/%s/%s",
+        g_get_home_dir(), GJAY_DIR, GJAY_FILE_DATA);
+  
     /* Cull songs which are no longer there */
-    for (llist = g_list_first(songs); llist; llist = g_list_next(llist)) {
+    for (llist = g_list_first(gjay->songs); llist; llist = g_list_next(llist)) {
         s = SONG(llist);
         if (!s->access_ok) {
             if (s->repeat_prev)
@@ -382,23 +407,24 @@ void write_data_file(void) {
         }
     }
     
-    f = fopen(buffer_temp, "w");
-    if (f) {
-        fprintf(f, "<gjay_data version=\"%s\">\n", GJAY_VERSION);
+    if ( (f = fopen(tmp_filename, "w")) == NULL) {
+      g_error(_("Unable to write song data %s\n"), tmp_filename);
+    } else {
+        fprintf(f, "<gjay_data version=\"%s\">\n", VERSION);
         for (llist = g_list_first(w_songs); llist; llist = g_list_next(llist))
             write_song_data(f, SONG(llist));
-        for (llist = g_list_first(not_songs); 
+        for (llist = g_list_first(gjay->not_songs); 
              llist; llist = g_list_next(llist))
             write_not_song_data(f, (char *) llist->data);
         fprintf(f, "</gjay_data>\n");
         fclose(f);
-        rename(buffer_temp, buffer);
-        songs_dirty = FALSE;
-    } else {
-        fprintf(stderr, "Unable to write song data %s\n", buffer_temp);
+        rename(tmp_filename, data_filename);
+        gjay->songs_dirty = FALSE;
     }
 
     g_list_free(w_songs);
+    g_free(tmp_filename);
+    g_free(data_filename);
 }
 
 
@@ -422,7 +448,8 @@ int append_daemon_file (song * s) {
         fclose(f);
         return file_seek;
     } else {
-        fprintf(stderr, "Error: unable to write %s.\nAnalysis for %s was skipped!\n", buffer, s->path);
+        g_warning(_("Unable to write '%s'.\nAnalysis for '%s' was skipped!\n"),
+            buffer, s->path);
     }
     return -1;
 }
@@ -444,7 +471,7 @@ int append_daemon_file (song * s) {
  *   <bpm>float</bpm>
  * </file>
  */
-static void write_song_data (FILE * f, song * s) {
+void write_song_data (FILE * f, song * s) {
     gchar * escape; /* Escape XML elements from text */
     int k;
 
@@ -529,7 +556,7 @@ void read_data_file ( void ) {
         snprintf(buffer, BUFFER_SIZE, "%s/%s/%s", 
                  getenv("HOME"), GJAY_DIR, files[k]);
         if (verbosity) {
-            printf("Reading from file %s\n", buffer);
+            printf(_("Reading from data file '%s'\n"), buffer);
         }
         f = fopen(buffer, "r");
         if (f) {
@@ -557,7 +584,7 @@ gboolean add_from_daemon_file_at_seek (int seek) {
         fseek(f, seek, SEEK_SET);
         result = read_data(f);
         if (result) {
-            songs_dirty = TRUE;
+            gjay->songs_dirty = TRUE;
         }
         fclose(f);
     }
@@ -607,7 +634,7 @@ void data_start_element  (GMarkupParseContext *context,
                           gpointer             user_data,
                           GError             **error) {
     song_parse_state * state = (song_parse_state *) user_data;
-    gchar * path, * repeat_path = NULL;
+    gchar * path = NULL, * repeat_path = NULL;
     song * original;
     element_type element;
     int k;
@@ -633,20 +660,20 @@ void data_start_element  (GMarkupParseContext *context,
         assert(path);
         
         if (state->not_song) {
-            if (!g_hash_table_lookup(not_song_hash, path)) {
+            if (!g_hash_table_lookup(gjay->not_song_hash, path)) {
                 state->new = TRUE;
                 /* Only keep track of files which still exist */
                 if (!access(path, R_OK)) {
                     path = g_strdup(path);
-                    not_songs = g_list_append(not_songs, path);
-                    g_hash_table_insert ( not_song_hash,
+                    gjay->not_songs = g_list_append(gjay->not_songs, path);
+                    g_hash_table_insert ( gjay->not_song_hash,
                                           path, 
                                           (gpointer) TRUE);
                 } 
             }
             return;
         }
-        state->s = g_hash_table_lookup(song_name_hash, path);
+        state->s = g_hash_table_lookup(gjay->song_name_hash, path);
         if (!state->s) {
             state->new = TRUE;
             state->s = create_song();
@@ -654,7 +681,7 @@ void data_start_element  (GMarkupParseContext *context,
         }
         if (repeat_path && (strlen(repeat_path) > 0)) {
             state->is_repeat = TRUE;
-            original = g_hash_table_lookup(song_name_hash, repeat_path);
+            original = g_hash_table_lookup(gjay->song_name_hash, repeat_path);
             assert(original);
             song_set_repeats(state->s, original);
         }
@@ -706,17 +733,17 @@ void data_end_element (GMarkupParseContext *context,
             latin1_path = strdup_to_latin1(state->s->path);
             state->s->access_ok = !access(latin1_path, R_OK);
             if (!state->s->access_ok) {
-                songs_dirty = TRUE;
+                gjay->songs_dirty = TRUE;
             }
             g_free(latin1_path);
 
             /* Add song to song list and hash table */
-            songs = g_list_append(songs, state->s);
-            g_hash_table_insert (song_name_hash,
+            gjay->songs = g_list_append(gjay->songs, state->s);
+            g_hash_table_insert (gjay->song_name_hash,
                                  state->s->path, 
                                  state->s);           
             hash_inode_dev(state->s, state->has_dev);
-            g_hash_table_insert (song_inode_dev_hash,
+            g_hash_table_insert (gjay->song_inode_dev_hash,
                                  &state->s->inode_dev_hash,
                                  state->s);
         }
@@ -814,9 +841,7 @@ static gboolean read_song_file_type ( char         * path,
                                       gchar       ** artist,
                                       gchar       ** album) {
     FILE * f; 
-    int i, result;
-    char * vf; // OggVorbis_Files
-    vorbis_comment * vc;
+    int result;
     struct stat buf;
     waveheaderstruct header;
     mp3info mp3;
@@ -827,44 +852,17 @@ static gboolean read_song_file_type ( char         * path,
     *title = NULL;
     
     switch (type) {
-    case OGG:
-        if (!gjay_vorbis_available()) 
-            return FALSE;
-        f = fopen(path, "r");
-        if (!f) 
-            return FALSE;
-        vf = malloc(2048); /* Much larger than OggVorbis_File struct on
-                              64 bit system */
-        if(gj_ov_open(f, vf, NULL, 0) == 0) {
-            vc = gj_ov_comment(vf, -1);
-            *length = gj_ov_time_total(vf, -1);
-            for (i = 0; i < vc->comments; i++) {
-                if (strncasecmp(vc->user_comments[i], "title=", 
-                                strlen("title=")) == 0) {
-                    *title = strdup_to_utf8(
-                        vc->user_comments[i] + strlen("title="));
-                } else if (strncasecmp(vc->user_comments[i], "artist=", 
-                                       strlen("artist=")) == 0) {
-                    *artist = strdup_to_utf8(
-                        vc->user_comments[i] + strlen("artist="));
-                } else if (strncasecmp(vc->user_comments[i], "album=", 
-                                       strlen("album=")) == 0) {
-                    *album = strdup_to_utf8(
-                        vc->user_comments[i] + strlen("album="));
-                }
-            }
-            gj_ov_clear(vf);
-            free(vf);
-            return TRUE;
-        }
-        fclose(f);
-        break;
     case MP3:
         bzero(&mp3, sizeof(mp3info));
         mp3.filename = path;
         mp3.file = fopen(path, "r");
+        if (!mp3.file) {
+          g_warning(_("Unable to read song data '%s' : %s\n"),
+              path, strerror(errno));
+          return FALSE;
+        }
         // returns 0 on success
-        result = get_mp3_info(&mp3, SCAN_QUICK, 0);
+        result = get_mp3_info(&mp3, SCAN_QUICK, 1);
         if (result == 0) {
             *length = mp3.seconds;
             if (mp3.id3_isvalid) {
@@ -891,7 +889,7 @@ static gboolean read_song_file_type ( char         * path,
         fread(&header, sizeof(waveheaderstruct), 1, f);
         wav_header_swab(&header);
         fclose(f);
-        if ((strncmp(header.chunk_type, "WAVE", 4) == 0) &&
+        if ((memcmp(header.chunk_type, "WAVE", 4) == 0) &&
             (header.byte_p_spl / header.modus == 2)) {
             stat(path, &buf);
             *length = (buf.st_size - sizeof(waveheaderstruct)) / 176758;
@@ -916,7 +914,7 @@ static int get_element ( gchar * element_name ) {
 
 
 int write_dirty_song_timeout ( gpointer data ) {
-    if (songs_dirty) 
+    if (gjay->songs_dirty) 
         write_data_file();
     return TRUE;
 }
@@ -935,25 +933,26 @@ gdouble song_force ( song * a, song  * b ) {
 
 /* Attraction is a value -1...1 for the affinity between A and B,
    with criteria weighed by prefs */
-gdouble song_attraction ( song * a, song  * b ) {
+gdouble song_attraction (song * a, song  * b ) {
     gdouble a_hue, a_saturation, a_brightness, a_freq, a_bpm, a_path;
     gdouble d, ba, bb, v_diff, a_max, attraction = 0;
     gint i;
+    GjayPrefs *prefs = gjay->prefs;
 
     a_max = 
-        (prefs.hue + 
-         prefs.brightness +
-         prefs.saturation +
-         prefs.freq + 
-         prefs.bpm +
-         prefs.path_weight);
+        (prefs->hue + 
+         prefs->brightness +
+         prefs->saturation +
+         prefs->freq + 
+         prefs->bpm +
+         prefs->path_weight);
     
-    a_hue = prefs.hue / a_max;
-    a_brightness = prefs.brightness / a_max;
-    a_saturation = prefs.saturation / a_max;
-    a_freq = prefs.freq / a_max;
-    a_bpm = prefs.bpm / a_max;
-    a_path = prefs.path_weight / a_max;
+    a_hue = prefs->hue / a_max;
+    a_brightness = prefs->brightness / a_max;
+    a_saturation = prefs->saturation / a_max;
+    a_freq = prefs->freq / a_max;
+    a_bpm = prefs->bpm / a_max;
+    a_path = prefs->path_weight / a_max;
 
     if (!(a->no_color || b->no_color)) {
         /* Hue is 0...6 */
@@ -1011,10 +1010,10 @@ gdouble song_attraction ( song * a, song  * b ) {
         attraction += d * a_freq;
     }
 
-    if (tree_depth && a->path && b->path) {
+    if (gjay->tree_depth && a->path && b->path) {
         d = explore_files_depth_distance(a->path, b->path);
         if (d >= 0) {
-            d = 1.0 - 2.0 * (d / tree_depth);
+            d = 1.0 - 2.0 * (d / gjay->tree_depth);
             /* d = -1 ... 1, where 1 is more similiar */
             attraction += d * a_path;
         }
@@ -1025,28 +1024,30 @@ gdouble song_attraction ( song * a, song  * b ) {
 
 
 /* Every song has a mass 0...1.0 */
-gdouble song_mass ( song * s ) {
-    gdouble max_mass = 0, song_mass = 0;
+static gdouble
+song_mass (song * s ) {
+  GjayPrefs *prefs = gjay->prefs;
+  gdouble max_mass = 0, song_mass = 0;
     max_mass = 
-        prefs.hue + 
-        prefs.brightness +
-        prefs.freq + 
-        prefs.bpm +
-        prefs.path_weight;
+        prefs->hue + 
+        prefs->brightness +
+        prefs->freq + 
+        prefs->bpm +
+        prefs->path_weight;
 
     assert(max_mass != 0);
 
-    song_mass += prefs.path_weight;
+    song_mass += prefs->path_weight;
     if (!s->no_data) {
-        song_mass += prefs.freq;
+        song_mass += prefs->freq;
     }
     if (!s->no_color) {
-        song_mass += prefs.hue;
-        song_mass += prefs.brightness;
-        song_mass += prefs.saturation;
+        song_mass += prefs->hue;
+        song_mass += prefs->brightness;
+        song_mass += prefs->saturation;
     }
     if (!s->bpm_undef) {
-        song_mass += prefs.bpm;
+        song_mass += prefs->bpm;
     }
     
     return (song_mass / max_mass);
