@@ -23,7 +23,7 @@
 #include <math.h>
 #include <string.h>
 #include "gjay.h"
-#include "gjay_xmms.h"
+#include "gjay_audacious.h"
 #include "ui.h"
 #include "rgbhsv.h"
 
@@ -35,9 +35,6 @@ enum {
    LAST_COLUMN
 };
 
-GList             * selected_songs = NULL; /* Songs which have been
-                                              selected */
-GList             * selected_files = NULL; /* List of selected file names */
 static GtkListStore * list_store;
 static GtkWidget    * icon, * play, * select_all_recursive;
 static GtkWidget    * label_name, * label_type;
@@ -70,7 +67,8 @@ static void     get_selected_color           ( HSV * color,
 #define COLOR_SWATCH_HEIGHT 20
 
 
-GtkWidget * make_selection_view ( void ) {
+GtkWidget *
+make_selection_view ( void ) {
     GtkWidget * vbox1, * vbox2, * hbox1, * hbox2;
     GtkWidget * alignment, * event_box, * swin;
     GtkCellRenderer * text_renderer, * pixbuf_renderer;
@@ -80,7 +78,7 @@ GtkWidget * make_selection_view ( void ) {
     
     hbox1 = gtk_hbox_new (FALSE, 2);
     gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 2);
-    icon = gtk_image_new_from_pixbuf (pixbufs[PM_ICON_CLOSED]);
+    icon = gtk_image_new_from_pixbuf (gjay->pixbufs[PM_ICON_CLOSED]);
     gtk_box_pack_start(GTK_BOX(hbox1), icon, FALSE, FALSE, 2);
 
     vbox2 = gtk_vbox_new(FALSE, 2);
@@ -105,9 +103,9 @@ GtkWidget * make_selection_view ( void ) {
     event_box = gtk_event_box_new ();
     gtk_box_pack_start(GTK_BOX(hbox2), event_box, FALSE, FALSE, 2);
 
-    play = gtk_image_new_from_pixbuf(pixbufs[PM_BUTTON_PLAY]);
-    gtk_tooltips_set_tip (tips, event_box,
-                          "Play the selected songs in XMMS", "");
+    play = gtk_image_new_from_pixbuf(gjay->pixbufs[PM_BUTTON_PLAY]);
+    gtk_widget_set_tooltip_text(event_box,
+        "Play the selected songs in the music player");
     gtk_container_add (GTK_CONTAINER(event_box), play);
     gtk_widget_set_events (event_box, GDK_BUTTON_PRESS_MASK);
     gtk_signal_connect (GTK_OBJECT(event_box), 
@@ -120,15 +118,15 @@ GtkWidget * make_selection_view ( void ) {
     
     event_box = gtk_event_box_new ();
     gtk_box_pack_start(GTK_BOX(hbox2), event_box, FALSE, FALSE, 2);
-    select_all_recursive = gtk_image_new_from_pixbuf(pixbufs[PM_BUTTON_ALL]);
-    gtk_tooltips_set_tip (tips, event_box,
-                          "Select all songs in this directory", "");
+    select_all_recursive = gtk_image_new_from_pixbuf(gjay->pixbufs[PM_BUTTON_ALL]);
+    gtk_widget_set_tooltip_text (event_box,
+        "Select all songs in this directory");
     gtk_container_add (GTK_CONTAINER(event_box), select_all_recursive);
     gtk_widget_set_events (event_box, GDK_BUTTON_PRESS_MASK);
     gtk_signal_connect (GTK_OBJECT(event_box), 
                         "button_press_event",
 			GTK_SIGNAL_FUNC (select_all_selected), 
-                        (gpointer *) TRUE);
+                        NULL);
     
     vbox_lower = gtk_vbox_new (FALSE, 2);  
     gtk_box_pack_start(GTK_BOX(vbox1), vbox_lower, TRUE, TRUE, 2);
@@ -172,7 +170,7 @@ GtkWidget * make_selection_view ( void ) {
     gtk_box_pack_start(GTK_BOX(vbox_lower), hbox2, FALSE, FALSE, 2);
 
     cwheel = create_colorwheel(COLORWHEEL_DIAMETER,
-                               &selected_songs,
+                               &(gjay->selected_songs),
                                update_selected_songs_color,
                                NULL);
 
@@ -211,13 +209,13 @@ GtkWidget * make_selection_view ( void ) {
 void set_selected_in_playlist_view ( gboolean in_view ) {
     gchar * fname;
     
-    if (selected_files == NULL)
+    if (gjay->selected_files == NULL)
         return;
-    if (g_list_length(selected_files) > 1)
+    if (g_list_length(gjay->selected_files) > 1)
         return;
-    fname = (gchar *) selected_files->data;
-    if ( g_hash_table_lookup(song_name_hash, fname) ||
-         g_hash_table_lookup(not_song_hash, fname))
+    fname = (gchar *) gjay->selected_files->data;
+    if ( g_hash_table_lookup(gjay->song_name_hash, fname) ||
+         g_hash_table_lookup(gjay->not_song_hash, fname))
         return;
     if (in_view) {
         gtk_widget_hide(select_all_recursive);
@@ -227,22 +225,24 @@ void set_selected_in_playlist_view ( gboolean in_view ) {
 }
 
 
-void set_selected_file ( char * file, 
-                         char * short_name, 
-                         gboolean is_dir ) {
+void
+set_selected_file ( char * file, 
+                    char * short_name, 
+                    gboolean is_dir )
+{
     char short_name_trunc[BUFFER_SIZE];
     GList * llist;
     int pm_type;
     song * s;
 
-    for (llist = g_list_first(selected_files); 
+    for (llist = g_list_first(gjay->selected_files); 
          llist;
          llist = g_list_next(llist))
         g_free(llist->data);
-    g_list_free(selected_files);
-    g_list_free(selected_songs);
-    selected_files = NULL;
-    selected_songs = NULL;
+    g_list_free(gjay->selected_files);
+    g_list_free(gjay->selected_songs);
+    gjay->selected_files = NULL;
+    gjay->selected_songs = NULL;
 
     if (file == NULL) {
         /* Hide everything */
@@ -262,25 +262,25 @@ void set_selected_file ( char * file,
     
     if (is_dir) {
         gtk_label_set_text(GTK_LABEL(label_name), short_name_trunc);
-        if (g_hash_table_lookup(new_song_dirs_hash, file)) {
+        if (g_hash_table_lookup(gjay->new_song_dirs_hash, file)) {
             gtk_image_set_from_pixbuf (GTK_IMAGE(icon), 
-                                       pixbufs[PM_ICON_CLOSED_NEW]);
+                                       gjay->pixbufs[PM_ICON_CLOSED_NEW]);
             gtk_label_set_text(GTK_LABEL(label_type), 
                                "Has uncategorized songs");
         } else {
             gtk_image_set_from_pixbuf (GTK_IMAGE(icon), 
-                                       pixbufs[PM_ICON_CLOSED]);
+                                       gjay->pixbufs[PM_ICON_CLOSED]);
             gtk_label_set_text(GTK_LABEL(label_type), "");
         }
         gtk_widget_hide(play);
 
         gtk_widget_show(select_all_recursive);
         gtk_widget_hide(vbox_lower);
-        selected_files = g_list_append(selected_files, g_strdup(file));
+        gjay->selected_files = g_list_append(gjay->selected_files, g_strdup(file));
     } else {
         gtk_widget_hide(select_all_recursive);
     
-        s = g_hash_table_lookup(song_name_hash, file);
+        s = g_hash_table_lookup(gjay->song_name_hash, file);
         
         if (s) {
             gtk_label_set_text(GTK_LABEL(label_name), "");
@@ -293,14 +293,14 @@ void set_selected_file ( char * file,
                 pm_type = PM_ICON_PENDING;
                 gtk_label_set_text(GTK_LABEL(label_type), "Will be analyzed");
             }
-            selected_songs = g_list_append(selected_songs, s);
+            gjay->selected_songs = g_list_append(gjay->selected_songs, s);
             gtk_widget_show(vbox_lower);
         } else {
             gtk_label_set_text(GTK_LABEL(label_name), short_name_trunc);
             gtk_widget_hide(play);
             gtk_widget_hide(vbox_lower);
 
-            if (g_hash_table_lookup(not_song_hash, file)) {
+            if (g_hash_table_lookup(gjay->not_song_hash, file)) {
                 pm_type = PM_ICON_NOSONG;
                 gtk_label_set_text(GTK_LABEL(label_type), "Not a song");
             } else {
@@ -310,8 +310,8 @@ void set_selected_file ( char * file,
         }
         
         gtk_image_set_from_pixbuf (GTK_IMAGE(icon), 
-                                   pixbufs[pm_type]);
-        selected_files = g_list_append(selected_files, g_strdup(file));
+                                   gjay->pixbufs[pm_type]);
+        gjay->selected_files = g_list_append(gjay->selected_files, g_strdup(file));
     }
     update_selection_area();
 }
@@ -327,21 +327,21 @@ static void set_selected_files (GList * files) {
     if (!files)
         return; 
 
-    for (llist = g_list_first(selected_files); 
+    for (llist = g_list_first(gjay->selected_files); 
          llist; 
          llist = g_list_next(llist)) {
         g_free(llist->data);
     }
-    g_list_free(selected_files);
-    g_list_free(selected_songs);
-    selected_songs = NULL;
-    selected_files = NULL;
+    g_list_free(gjay->selected_files);
+    g_list_free(gjay->selected_songs);
+    gjay->selected_songs = NULL;
+    gjay->selected_files = NULL;
 
     for (llist = g_list_first(files); llist; llist = g_list_next(llist)) {
-        if (g_hash_table_lookup(not_song_hash, llist->data)) {
+        if (g_hash_table_lookup(gjay->not_song_hash, llist->data)) {
             g_free(llist->data);
         } else {
-            s = g_hash_table_lookup(song_name_hash, llist->data);
+            s = g_hash_table_lookup(gjay->song_name_hash, llist->data);
             if (!s) {
                 /* This may happen a directory contains an empty directory, 
                    so the file list includes a directory path and not
@@ -352,8 +352,8 @@ static void set_selected_files (GList * files) {
                     gdk_pixbuf_unref(s->freq_pixbuf);
                     s->freq_pixbuf = NULL;
                 }
-                selected_songs = g_list_append(selected_songs, s);
-                selected_files = g_list_append(selected_files, llist->data);
+                gjay->selected_songs = g_list_append(gjay->selected_songs, s);
+                gjay->selected_files = g_list_append(gjay->selected_files, llist->data);
             } 
         }
     }
@@ -363,29 +363,33 @@ static void set_selected_files (GList * files) {
     gtk_widget_show(icon);
     gtk_widget_show(vbox_lower);
     gtk_label_set_text(GTK_LABEL(label_type), "Contains...");
-    gtk_image_set_from_pixbuf (GTK_IMAGE(icon), pixbufs[PM_ICON_OPEN]);
+    gtk_image_set_from_pixbuf (GTK_IMAGE(icon), gjay->pixbufs[PM_ICON_OPEN]);
     update_selection_area();
 }
 
 
-static gboolean play_selected (GtkWidget *widget,
-                               GdkEventButton *event,
-                               gpointer user_data) {
-    play_songs(selected_songs);
-    return TRUE;
+static gboolean
+play_selected (GtkWidget *widget,
+               GdkEventButton *event,
+               gpointer user_data)
+{
+  play_songs(gjay->selected_songs);
+  return TRUE;
 }
 
 
-static gboolean select_all_selected (GtkWidget *widget,
-                                     GdkEventButton *event,
-                                     gpointer user_data) {
-    GList * list;
-    if (selected_files) {
-        list = explore_files_in_dir(g_list_first(selected_files)->data, 
-                                    (gboolean) user_data);
-        set_selected_files(list);
-    }
-    return TRUE;
+static gboolean
+select_all_selected (GtkWidget *widget,
+                     GdkEventButton *event,
+                     gpointer user_data)
+{
+  GList * list;
+
+  if (gjay->selected_files) {
+      list = explore_files_in_dir(g_list_first(gjay->selected_files)->data, TRUE);
+      set_selected_files(list);
+  }
+  return TRUE;
 }
 
 
@@ -411,7 +415,7 @@ void populate_selected_list (void) {
     GtkTreeIter iter;
 
     gtk_list_store_clear (GTK_LIST_STORE(list_store)); 
-    for (llist = g_list_first(selected_songs); 
+    for (llist = g_list_first(gjay->selected_songs); 
          llist; 
          llist = g_list_next(llist)) {
         s = (song *) llist->data;
@@ -451,15 +455,15 @@ void update_selected_songs_color (gpointer data,
     gchar * dir = NULL;
     HSV hsv;
 
-    if (!selected_songs)
+    if (!gjay->selected_songs)
         return;
 
     hsv = get_colorwheel_color(GTK_WIDGET(data));
 
-    s = SONG(selected_songs);
+    s = SONG(gjay->selected_songs);
     dir = parent_dir(s->path);
 
-    for (llist = g_list_first(selected_songs); llist; 
+    for (llist = g_list_first(gjay->selected_songs); llist; 
          llist = g_list_next(llist)) {
         s = (song *) llist->data;
 
@@ -481,7 +485,7 @@ void update_selected_songs_color (gpointer data,
             update_song_has_rating_color(s);
     }
 
-    songs_dirty = TRUE;
+    gjay->songs_dirty = TRUE;
 }
 
 
@@ -494,7 +498,7 @@ void redraw_rating (void) {
     lower = MAX_RATING + 1;
     upper = -1;
     total = 0;
-    for (n = 0, k = 0, llist = g_list_first(selected_songs);
+    for (n = 0, k = 0, llist = g_list_first(gjay->selected_songs);
          llist; 
          llist = g_list_next(llist)) {
         s = (song *) llist->data;
@@ -528,7 +532,7 @@ static void rating_changed ( GtkRange *range,
 
     val = gtk_range_get_value(range);
 
-    for (llist = g_list_first(selected_songs); llist; 
+    for (llist = g_list_first(gjay->selected_songs); llist; 
          llist = g_list_next(llist)) {
         s = (song *) llist->data;
 
@@ -551,7 +555,7 @@ static void rating_changed ( GtkRange *range,
     }
     gtk_label_set_text (GTK_LABEL(label_rating), "Rating");
 
-    songs_dirty = TRUE;
+    gjay->songs_dirty = TRUE;
 }
 
 
@@ -573,7 +577,7 @@ static void update_dir_has_rating_color ( gchar * dir ) {
     gchar * parent;
     gchar * str;
     
-    str = g_hash_table_lookup(new_song_dirs_hash, dir);
+    str = g_hash_table_lookup(gjay->new_song_dirs_hash, dir);
     if (!str) 
         return;
 
@@ -584,8 +588,8 @@ static void update_dir_has_rating_color ( gchar * dir ) {
        icon, remove info */
     explore_update_path_pm(dir, PM_DIR_CLOSED);
     
-    g_hash_table_remove(new_song_dirs_hash, str);
-    new_song_dirs = g_list_remove(new_song_dirs, str);
+    g_hash_table_remove(gjay->new_song_dirs_hash, str);
+    gjay->new_song_dirs = g_list_remove(gjay->new_song_dirs, str);
     g_free(str);
     
     /* Check the parent directory */
@@ -615,7 +619,7 @@ void get_selected_color (HSV * color,
     assert(color && no_color && multiple_colors);
     *no_color = TRUE;
     bzero(&rgb_sum, sizeof(RGB));
-    for (llist = g_list_first(selected_songs);
+    for (llist = g_list_first(gjay->selected_songs);
          llist;
          llist = g_list_next(llist)) {
         s = (song *) llist->data;
